class CurveDrawer {
	constructor()
	{
		this.prog   = InitShaderProgram( curvesVS, curvesFS );
		//get uniform and attrib locations
		this.t = gl.getAttribLocation(this.prog, 't');
		this.mvp = gl.getUniformLocation(this.prog, 'mvp');
		this.p0 = gl.getUniformLocation(this.prog, 'p0');
		this.p1 = gl.getUniformLocation(this.prog, 'p1');
		this.p2 = gl.getUniformLocation(this.prog, 'p2');
		this.p3 = gl.getUniformLocation(this.prog, 'p3');
		// Initialize the attribute buffer
		this.steps = 100;
		var tv = [];
		for ( var i=0; i<this.steps; ++i ) {
			tv.push( i / (this.steps-1) );
		}
		//bind attribute buffer
		gl.useProgram( this.prog );
		this.buffer = gl.createBuffer();
		gl.bindBuffer(gl.ARRAY_BUFFER, this.buffer);
		gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(tv), gl.STATIC_DRAW);
	}
	setViewport( width, height )
	{
		gl.useProgram( this.prog );
		var trans = [ 2/width,0,0,0,  0,-2/height,0,0, 0,0,1,0, -1,1,0,1 ];
		gl.uniformMatrix4fv( this.mvp, false, trans );
	}
	updatePoints( pt )
	{
		gl.useProgram(this.prog);
		gl.uniform2f(this.p0, pt[0].getAttribute("cx"), pt[0].getAttribute("cy"));
		gl.uniform2f(this.p1, pt[1].getAttribute("cx"), pt[1].getAttribute("cy"));
		gl.uniform2f(this.p2, pt[2].getAttribute("cx"), pt[2].getAttribute("cy"));
		gl.uniform2f(this.p3, pt[3].getAttribute("cx"), pt[3].getAttribute("cy"));
	}
	draw()
	{
		gl.useProgram( this.prog );
		gl.bindBuffer( gl.ARRAY_BUFFER, this.buffer );
		gl.vertexAttribPointer( this.t, 1, gl.FLOAT, false, 0, 0 );
		gl.enableVertexAttribArray( this.t );
		gl.drawArrays( gl.LINE_STRIP, 0, 100 );
	}
}

// Vertex Shader
var curvesVS = `
	attribute float t;
	uniform mat4 mvp;
	uniform vec2 p0;
	uniform vec2 p1;
	uniform vec2 p2;
	uniform vec2 p3;
	void main()
	{
		vec2 p = ((1.0-t)*(1.0-t)*(1.0-t) * p0) +
				(3.0 * (1.0-t) * (1.0-t) * t * p1) +
				(3.0 * (1.0-t) * (t)*(t) * p2) +
				((t)*(t)*(t) * p3);

		gl_Position = mvp * vec4(p,0,1);
	}
`;

// Fragment Shader
var curvesFS = `
	precision mediump float;
	void main()
	{
		gl_FragColor = vec4(1,0,0,1);
	}
`;
